package ovh.herisson.Clyde.EndPoints;

import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Repositories.*;
import ovh.herisson.Clyde.Repositories.Inscription.ExternalCurriculumRepository;
import ovh.herisson.Clyde.Repositories.Inscription.MinervalRepository;
import ovh.herisson.Clyde.Repositories.Inscription.ScholarshipRequestRepository;
import ovh.herisson.Clyde.Services.*;
import ovh.herisson.Clyde.Services.ScientificPublications.ResearchesService;
import ovh.herisson.Clyde.Tables.*;
import ovh.herisson.Clyde.Tables.ScientificPublications.Access;
import ovh.herisson.Clyde.Tables.ScientificPublications.PaperType;
import ovh.herisson.Clyde.Tables.ScientificPublications.Research;
import ovh.herisson.Clyde.Tables.ScientificPublications.Researcher;
import ovh.herisson.Clyde.Services.Inscription.InscriptionService;
import ovh.herisson.Clyde.Tables.Inscription.ExternalCurriculum;
import ovh.herisson.Clyde.Tables.Inscription.InscriptionRequest;
import ovh.herisson.Clyde.Tables.Inscription.Minerval;
import ovh.herisson.Clyde.Tables.Inscription.ScholarshipRequest;

import java.util.*;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
public class MockController {
    private final BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public final UserRepository userRepo;
    public final TokenRepository tokenRepo;
    public final TokenService tokenService;
    public final CurriculumCourseService CurriculumCourseService;
    public final CurriculumService curriculumService;
    public final CourseService courseService;
    public final ExternalCurriculumRepository externalCurriculumRepository;
    public final InscriptionService inscriptionService;

    public final ResearchesService researchesService;
    public final UserCurriculumRepository ucr;
    public final MinervalRepository minervalRepository;
    public final ScholarshipRequestRepository scholarshipRequestRepository;

    ArrayList<User> mockUsers;
    public MockController(UserRepository userRepo, TokenRepository tokenRepo, TokenService tokenService, CurriculumCourseService CurriculumCourseService, CurriculumService curriculumService, CourseService courseService, ExternalCurriculumRepository externalCurriculumRepository,  ResearchesService researchesService, InscriptionService inscriptionService, UserCurriculumRepository ucr, MinervalRepository minervalRepository, ScholarshipRequestRepository scholarshipRequestRepository){
        this.tokenRepo = tokenRepo;
        this.userRepo = userRepo;
        this.tokenService = tokenService;
        this.CurriculumCourseService = CurriculumCourseService;
        this.curriculumService = curriculumService;
        this.courseService = courseService;
        this.externalCurriculumRepository = externalCurriculumRepository;
        this.inscriptionService = inscriptionService;
        this.researchesService = researchesService;
        this.ucr = ucr;
        this.minervalRepository = minervalRepository;
        this.scholarshipRequestRepository = scholarshipRequestRepository;
    }

    /** Saves an example of each user type by :
     * email : FooRole@FooRole.com, password : FooRole and token : FooRole
     * For example the admin as "admin@admin.com" as email and "admin" as both password and token
     * They all have silly names
     */

    @PostMapping("/mock")
    public void postMock() {

            // user part
            User herobrine = new User("brine", "hero", "admin@admin.com", "behind", "ShadowsLand", new Date(0), null, Role.Admin, passwordEncoder.encode("admin"));
            User joe = new User("Mama", "Joe", "student@student.com", "roundabout", "England", new Date(0), null, Role.Student, passwordEncoder.encode("student"));
            User meh = new User("Polo", "Marco", "secretary@secretary.com", "a Box", "Monaco", new Date(0), null, Role.Secretary, passwordEncoder.encode("secretary"));
            User joke = new User("Gaillard", "Corentin", "teacher@teacher.com", "lab", "faculty", new Date(0), null, Role.Teacher, passwordEncoder.encode("teacher"));
            User jojo = new User("Bridoux", "Justin", "teacher2@teacher2.com", "lab", "faculty", new Date(0), null, Role.Teacher, passwordEncoder.encode("teacher"));
            User lena = new User("Louille", "Lena", "inscriptionService@InscriptionService.com", "no", "yes", new Date(0), null, Role.InscriptionService, passwordEncoder.encode("inscriptionService"));
            User popo = new User("Smith", "Paul", "paulsmith@gmail.com", "306 rue du poulet", "belgique", new Date(0), null, Role.Student, passwordEncoder.encode("jesuispaulleroi"));
            mockUsers = new ArrayList<>(Arrays.asList(herobrine, joe, meh, joke, lena, jojo, popo));

            userRepo.saveAll(mockUsers);

            Minerval minerval = new Minerval(joe.getRegNo(), 0, 852, 2023);
            minervalRepository.save(minerval);
            // Course / Curriculum part

            Curriculum infoBab1 = new Curriculum(1, "info");
            Curriculum chemistryBab1 = new Curriculum(1, "chemistry");
            Curriculum psychologyBab1 = new Curriculum(1, "psychology");
            Curriculum infoBab2 = new Curriculum(2, "info");
            Curriculum masterinfo1 = new Curriculum(4, "info");
            Curriculum masterinfo2 = new Curriculum(5, "info");

            curriculumService.save(infoBab1);
            curriculumService.save(chemistryBab1);
            curriculumService.save(psychologyBab1);
            curriculumService.save(infoBab2);
            curriculumService.save(masterinfo1);
            curriculumService.save(masterinfo2);

            ucr.save(new UserCurriculum(joe, infoBab1, 2022));
            ucr.save(new UserCurriculum(joe, chemistryBab1, 2023));
            ucr.save(new UserCurriculum(joe, infoBab1, 2023));
            ucr.save(new UserCurriculum(joe, psychologyBab1, 2020));
            ucr.save(new UserCurriculum(popo, infoBab1, 2022));
            ucr.save(new UserCurriculum(popo, infoBab2, 2023));

            Course progra1 = new Course(5, "Programmation et algorithmique 1", joke);
            Course chemistry1 = new Course(12, "Thermochimie", joke);
            Course psycho1 = new Course(21, "Neuroreaction of isolated brain cells", joke);
            Course commun = new Course(2, "cours commun", joke);

            courseService.save(progra1);
            courseService.save(chemistry1);
            courseService.save(psycho1);
            courseService.save(commun);

            ScholarshipRequest ssr1 = new ScholarshipRequest(joe, RequestState.Pending, 0, new Date(), "test", "test");
            scholarshipRequestRepository.save(ssr1);

            CurriculumCourseService.save(new CurriculumCourse(infoBab1, progra1));
            CurriculumCourseService.save(new CurriculumCourse(infoBab1, commun));
            CurriculumCourseService.save(new CurriculumCourse(infoBab1, psycho1));
            CurriculumCourseService.save(new CurriculumCourse(psychologyBab1, psycho1));
            CurriculumCourseService.save(new CurriculumCourse(psychologyBab1, commun));


            CurriculumCourseService.save(new CurriculumCourse(chemistryBab1, commun));
            CurriculumCourseService.save(new CurriculumCourse(chemistryBab1, chemistry1));


            InscriptionRequest inscriptionRequest = new InscriptionRequest("helen", "prenom", "non", "helen@gmail.com", "america", new Date(), (long) 4, RequestState.Pending, "yes.png", "password", null, new Date(), RequestState.Pending);

            inscriptionService.save(inscriptionRequest);
            ExternalCurriculum externalCurriculum = new ExternalCurriculum(inscriptionRequest, "HEH", "Bachelier en informatique", "Completed", 2015, 2018, null, null);
            externalCurriculumRepository.save(externalCurriculum);

            ///////////////////////////
            // extension Publications Scientifiques
            Researcher jojoResearcherAccount = new Researcher(jojo, "3363-22555-AB33-T", null, "IT");

            Researcher joResearchAccount = new Researcher(joe,"N555-321213-BED-DD",null, "Physics");


            Researcher output = researchesService.saveResearcher(jojoResearcherAccount);
            Researcher joOutput = researchesService.saveResearcher(joResearchAccount);

            Set<Researcher> coAuthor = new HashSet<>();
            coAuthor.add(joOutput);

            Research jojoResearch = new Research("Graphs : Advanced Search Algorithms", output, new Date(0),
                    PaperType.Article, "test.pdf", null, "english",
                    Access.OpenSource, "IT", "This Article's title speaks for itself \n We'll discuss about advanced Graph search Algorithms",coAuthor);
            Research restrictedResearch = new Research("just another Name", output, new Date(1111111111),
                    PaperType.Article, "restricted", null, "english",
                    Access.Restricted, "Restricted", "This Article's title speaks for itself\n We'll discuss about advanced Graph search Algorithms", new HashSet<>());

            Research privateResearch = new Research("the great Potato War", output, new Date(),
                    PaperType.Article, "private", null, "english",
                    Access.Private, "private", "This Article's title speaks for itself\n We'll discuss about advanced Graph search Algorithms",null);


            researchesService.saveResearch(restrictedResearch);
            researchesService.saveResearch(privateResearch);
            researchesService.saveResearch(jojoResearch);

    }
}

